@@ -1,4 +1,5 @@
 # Copyright (C) 2004-2006 Laurent Sansonetti
+# Copyright (C) 2009 Cathal Mc Ginley
 #
 # Alexandria is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -32,6 +33,8 @@ module Alexandria
 
     class SearchError < StandardError; end
     class NoResultsError < SearchError; end
+    class ProviderSkippedError < NoResultsError; end # not an error :^(
+    class SearchEmptyError < SearchError; end # sigh, again not really an error
     class TooManyResultsError < SearchError; end
     class InvalidSearchTypeError < SearchError; end
 
@@ -42,10 +45,17 @@ module Alexandria
       begin
         factory = self.instance[factory_n]
         puts factory.fullname + " lookup" if $DEBUG
+        if (not factory.enabled)
+          puts factory.fullname + " disabled!, skipping..." if $DEBUG
+          raise ProviderSkippedError
+        end
         self.instance.changed
         self.instance.notify_observers(:searching, factory.fullname) # new
         results = factory.search(criterion, type)
 
+        # sanity check if at least one valid result is actually found
+        results.delete_if { |book, cover| book.nil? }
+
         if results.length == 0
           self.instance.changed
           self.instance.notify_observers(:not_found, factory.fullname) # new
@@ -57,16 +67,22 @@ module Alexandria
           return results
         end
       rescue Exception => boom
-        unless boom.instance_of? NoResultsError
+        if boom.kind_of? NoResultsError
+          unless boom.instance_of? ProviderSkippedError
+            self.instance.changed
+            self.instance.notify_observers(:not_found, factory.fullname) # new
+            Thread.new {sleep(0.5)}.join
+          end
+        else        
           self.instance.changed
           self.instance.notify_observers(:error, factory.fullname) # new
           Thread.new {sleep(0.5)}.join # hrmmmm, to make readable...
-          trace = boom.backtrace.join("\n> ")
+          trace = boom.backtrace.join("\n >")
           log.warn { "Provider #{factory.name} encountered error: #{boom.message} #{trace}" }
         end
         if self.last == factory
           log.warn { "Error while searching #{criterion}" }
-          raise case boom
+          message = case boom
                 when Timeout::Error
                   _("Couldn't reach the provider '%s': timeout " +
                     "expired.") % factory.name
@@ -80,6 +96,11 @@ module Alexandria
                     "search criterion is spelled correctly, and " +
                     "try again.")
 
+                when ProviderSkippedError
+                  _("No results were found.  Make sure your " +
+                    "search criterion is spelled correctly, and " +
+                    "try again.")
+
                 when TooManyResultsError
                   _("Too many results for that search.")
 
@@ -89,6 +110,8 @@ module Alexandria
                 else
                   boom.message
                 end
+          puts "raising empty error #{message}"
+          raise SearchEmptyError, message
         else
           factory_n += 1
           retry
@@ -169,6 +192,7 @@ module Alexandria
     end
 
     class AbstractProvider
+      include GetText
       attr_reader :prefs
       attr_accessor :name, :fullname
 
@@ -176,6 +200,16 @@ module Alexandria
         @name = name
         @fullname = (fullname or name)
         @prefs = Preferences.new(self)
+        @prefs.add("enabled", _("Enabled"), true, [true,false])
+      end
+      
+      def enabled()
+        @prefs['enabled']
+      end
+      
+      def toggle_enabled()
+        old_value = enabled()
+        @prefs.variable_named('enabled').new_value = (not old_value)
       end
 
       def reinitialize(fullname)
@@ -223,7 +257,7 @@ module Alexandria
       def transport
         config = Alexandria::Preferences.instance.http_proxy_config
         config ? Net::HTTP.Proxy(*config) : Net::HTTP
-      end
+      end    
 
       def abstract?
         self.class.abstract?
@@ -250,45 +284,21 @@ module Alexandria
       unabstract
     end
 
-    #require 'alexandria/book_providers/bn'
-    log.info { "Not loading Barnes and Noble (provider not functional)" }
+    class WebsiteBasedProvider < GenericProvider
+      # further defined in alexandria/book_providers/web.rb
+      # its implementation requires Hpricot and HTMLEntities
+    end
 
-    require 'alexandria/book_providers/proxis'
-
-    require 'alexandria/book_providers/mcu' #?
-
-    #require 'alexandria/book_providers/thalia'
-    log.info { "Not loading Thalia (provider not functional)" }
+    require 'alexandria/book_providers/mcu' # yep, still mostly works !
+    require 'alexandria/book_providers/douban' # only requires YAML
 
     #require 'alexandria/book_providers/ibs_it'
-    log.info { "Not loading IBS (provider not functional)" }
-
     #require 'alexandria/book_providers/renaud'
-    log.info { "Not loading Renaud (provider not functional)" }
-
-
-    #require 'alexandria/book_providers/adlibris' # new, now needs other deps
-    ## require 'alexandria/book_providers/ls' # obsolete, replaced by siciliano
     #require 'alexandria/book_providers/bol_it'
-    log.info { "Not loading BOL (provider not functional)" }
-
     #require 'alexandria/book_providers/webster_it'
-    log.info { "Not loading Webster (provider not functional)" }
+    log.info { "Not loading IBS, Renaud, BOL, Webster (providers not functional)" }
 
-    #require 'alexandria/book_providers/worldcat' # new, now needs hpricot
 
-    # mechanize is optional
-    #begin
-    #  begin
-    #    require 'mechanize'
-    #  rescue LoadError
-    #    require 'rubygems'
-    #    require 'mechanize'
-    #  end
-    #  require 'alexandria/book_providers/dea_store_it'
-    #rescue LoadError
-    #  log.warn { "Can't load mechanize, hence provider Deastore not available" }
-    #end
 
     # Amazon AWS (Amazon Associates Web Services) provider, needs hpricot
     begin
@@ -299,16 +309,13 @@ module Alexandria
         require 'hpricot'
       end
       require 'alexandria/book_providers/amazon_aws'
-      require 'alexandria/book_providers/deastore'
-      require 'alexandria/book_providers/siciliano'
-      require 'alexandria/book_providers/worldcat'
-    rescue LoadError
-      log.warn { "Can't load hpricot, hence provider Amazon not available" }
-      log.warn { "Can't load hpricot, hence provider DeaStore not available" }
-      log.warn { "Can't load hpricot, hence provider Siciliano not available" }
-      log.warn { "Can't load hpricot, hence provider WorldCat not available" }
+    rescue LoadError => ex
+      log.error { ex }
+      log.error { ex.backtrace.join("\n> ") }
+      log.warn { "Can't load 'hpricot', hence Amazon book provider will not be available" }
     end
 
+    
     # AdLibris (needs htmlentities and hpricot)
     begin
       begin
@@ -319,19 +326,37 @@ module Alexandria
         require 'htmlentities'
         require 'hpricot'
       end
+      
+      require 'alexandria/book_providers/web'
       require 'alexandria/book_providers/adlibris'
+      require 'alexandria/book_providers/barnes_and_noble'
+      require 'alexandria/book_providers/deastore'
+      require 'alexandria/book_providers/proxis'
+      require 'alexandria/book_providers/siciliano'
+      require 'alexandria/book_providers/thalia'
+      require 'alexandria/book_providers/worldcat'
+
     rescue LoadError => ex
-      log.warn { "Can't load provider AdLibris, requires both hpricot and htmlentities: #{ex.message}" }
+      log.warn { "Can't load 'hpricot' and 'htmlentities', hence AdLibris, Barnes & Noble, DeaStore, Proxis, Siciliano, Thalia and Worldcat book providers will not be available" }
     end
 
 
     # Ruby/ZOOM is optional
     begin
+      begin
+        require 'zoom'
+        require 'marc'
+      rescue LoadError
+        require 'rubygems'
+        require 'zoom'
+        require 'marc'
+      end
       require 'alexandria/book_providers/z3950'
     rescue LoadError
       log.info { "Can't load Ruby/ZOOM, hence Z39.50 and providers Library of Congress, British Library not available" }
     end
 
+
     attr_reader :abstract_classes
 
     def initialize
@@ -352,6 +377,7 @@ module Alexandria
         klass = self.class.module_eval(constant)
         if klass.ancestors.include?(AbstractProvider) and
             klass != GenericProvider and
+            klass != WebsiteBasedProvider and
             klass != AbstractProvider
 
           if klass.abstract?
