@@ -23,13 +23,17 @@
 # New WorldCat provider, taken from the Palatina MetaDataSource and
 # modified to fit the structure of Alexandria book providers.
 # (25 Feb 2009)
+#
+# Updated from Palatina, to reflect changes in the worldcat website.
+# (1 Sep 2009)
+
 
 require 'cgi'
 require 'alexandria/net'
 
 module Alexandria
   class BookProviders
-    class WorldCatProvider < GenericProvider
+    class WorldCatProvider < WebsiteBasedProvider
       include Alexandria::Logging
 
       SITE = "http://www.worldcat.org"
@@ -37,17 +41,20 @@ module Alexandria
 
       def initialize()
         super("WorldCat", "WorldCat")
+        #prefs.add("enabled", _("Enabled"), true, [true,false])
+        prefs.read
       end
 
       def search(criterion, type)
-        puts create_search_uri(type, criterion)
+        #puts create_search_uri(type, criterion)
         req = create_search_uri(type, criterion)
+        puts req if $DEBUG
         html_data = transport.get_response(URI.parse(req))
         # Note: I tried to use Alexandria::WWWAgent, 
         #       but this caused failures here (empty pages...)
         #       find out how the requests differ
 
-        puts html_data.class
+        #puts html_data.class
         if type == SEARCH_BY_ISBN
           parse_result_data(html_data.body, criterion)
         else
@@ -96,11 +103,11 @@ module Alexandria
 
 
       def parse_search_result_data(html)
-        doc = Hpricot(html)
+        doc = html_to_doc(html)
         book_search_results = []
         begin
           result_cells = doc/'td.result/div.name/..'
-          puts result_cells.length
+          #puts result_cells.length
           result_cells.each do |td|
             type_icon = (td%'div.type/img.icn')
             next unless (type_icon and type_icon['src'] =~ /icon-bks/)
@@ -127,8 +134,8 @@ module Alexandria
 
 
 
-    def parse_result_data(html, search_isbn=nil)
-      doc = Hpricot(html)
+    def parse_result_data(html, search_isbn=nil, recursing=false)
+      doc = html_to_doc(html)
       
       begin
         if doc%'div#div-results-none'
@@ -136,7 +143,51 @@ module Alexandria
           raise NoResultsError
         end
 
-        title_header = doc%'h1.item-title'
+
+        if doc % 'table.table-results'
+          if recursing
+            log.warn { "Infinite loop prevented redirecting through WorldCat" }
+            raise NoResultsError
+          end
+          log.info { "Found multiple results for lookup: checking each" }
+          search_results = parse_search_result_data(html)
+          book = nil
+          cover_url = nil
+          first_result = nil
+          search_results.each do |rslt|
+            #rslt = search_results.rslt
+            log.debug { "checking #{rslt[:url]}" }
+            rslt2 = transport.get_response(URI.parse(rslt[:url]))
+            html2 = rslt2.body
+
+            book,cover_url = parse_result_data(html2, search_isbn, true)
+            if first_result.nil?
+              first_result = [book, cover_url]
+            end
+
+            log.debug { "got book #{book}" }
+
+            if search_isbn
+              search_isbn_canon = Library.canonicalise_ean(search_isbn)
+              rslt_isbn_canon = Library.canonicalise_ean(book.isbn)
+              if search_isbn_canon == rslt_isbn_canon
+                log.info { "book #{book} is a match"}
+                return [book, cover_url]
+              end
+              log.debug {"not a match, checking next"}
+            else
+              # no constraint to match isbn, just return first result
+              return [book, cover_url]
+            end
+          end
+          
+          # gone through all and no ISBN match, so just return first result
+          log.info {"no more results to check. Returning first result, just an approximation"}
+          return first_result
+
+        end
+
+        title_header = doc%'h1.title'
         title = title_header.inner_text if title_header
         unless title
           log.warn { "Unexpected lack of title from WorldCat lookup" }
@@ -145,18 +196,29 @@ module Alexandria
         log.info { "Found book #{title} at WorldCat" }
 
         authors = []
-        authors_div = doc%'div.item-author'
-        if authors_div
-          (authors_div/:a).each do |a|
+        authors_tr = doc%'tr#details-allauthors'
+        if authors_tr
+          (authors_tr/:a).each do |a|
             authors << a.inner_text
           end
         end
 
         # can we do better? get the City name?? or multiple publishers?
-        publisher_row = doc%'td.label[text()*=Publisher]/..'
+        bibdata = doc % 'div#bibdata'
+        bibdata_table = bibdata % :table
+        publisher_row = bibdata_table % 'th[text()*=Publisher]/..'
+
         if publisher_row
           publication_info = (publisher_row/'td').last.inner_text
-          publication_info =~ /:*([^;,]+)/
+
+          if publication_info.index(';')
+            publication_info =~ /;[\s]*([^\d]+)[\s]*[\d]*/
+          elsif publication_info.index(':')
+            publication_info =~ /:[\s]*([^;:,]+)/
+          else
+            publication_info =~ /([^;,]+)/
+          end
+
           publisher = $1
           publication_info =~ /([12][0-9]{3})/
           year = $1.to_i if $1
@@ -167,7 +229,7 @@ module Alexandria
 
         isbn = search_isbn
         unless isbn
-          isbn_row = doc%'td.label[text()*=ISBN]/..'
+          isbn_row = doc % 'tr#details-standardno' ##bibdata_table % 'th[text()*=ISBN]/..'
           if isbn_row
             isbns = (isbn_row/'td').last.inner_text.split
             isbn = Library.canonicalise_isbn(isbns.first)
