@@ -114,7 +114,15 @@ module Alexandria
       end
 
       def on_export widget, event
-        ExportDialog.new(@main_app, selected_library, library_sort_order)
+        begin
+          ExportDialog.new(@main_app, selected_library, library_sort_order)
+        rescue Exception => ex
+          log.error { "problem with immediate export #{ex} try again" }
+
+          ErrorDialog.new(@main_app, _("Export failed"),
+                              _("Try letting this library load " + 
+                                "completely before exporting."))
+        end
       end
 
       def on_acquire widget, event
@@ -130,7 +138,7 @@ module Alexandria
       end
 
       def on_properties widget, event
-        if @library_listview.focus?
+        if @library_listview.focus? or selected_books.empty?
           library = selected_library
           if library.is_a?(SmartLibrary)
             SmartLibraryPropertiesDialog.new(@main_app, library) do
@@ -210,10 +218,16 @@ module Alexandria
 
       def on_delete widget, event
         library = selected_library
-        books = @library_listview.focus? ? nil : selected_books
+
+        if selected_books.empty?
+          books = nil
+        else
+          books = selected_books
+        end
+        #books = @library_listview.focus? ? nil : selected_books
         is_smart = library.is_a?(SmartLibrary)
         last_library = (@libraries.all_regular_libraries.length == 1)
-        if (!is_smart && last_library)
+        if (books.nil? && !is_smart && last_library)
           log.warn { "Attempted to delete last library, fix GUI" }
           return
         end
@@ -319,12 +333,12 @@ module Alexandria
         toggle_actions = [
           ["Sidepane", nil, _("Side _Pane"), "F9", nil,
             on_view_sidepane, true],
-            ["Toolbar", nil, _("_Toolbar"), nil, nil,
-              on_view_toolbar, true],
-              ["Statusbar", nil, _("_Statusbar"), nil, nil,
-                on_view_statusbar, true],
-                ["ReversedOrder", nil, _("Re_versed Order"), nil, nil,
-                  on_reverse_order],
+          ["Toolbar", nil, _("_Toolbar"), nil, nil,
+            on_view_toolbar, true],
+          ["Statusbar", nil, _("_Statusbar"), nil, nil,
+            on_view_statusbar, true],
+          ["ReversedOrder", nil, _("Re_versed Order"), nil, nil,
+            on_reverse_order],
         ]
 
         view_as_actions = [
@@ -361,6 +375,7 @@ module Alexandria
         end
         @actiongroup.add_radio_actions(arrange_icons_actions) do |action, current|
           @prefs.arrange_icons_mode = current.current_value
+          setup_books_iconview_sorting
         end
       end
     end
