@@ -18,6 +18,7 @@
 
 #require 'monitor'
 require 'alexandria/scanners/cuecat'
+require 'alexandria/scanners/keyboard'
 
 module Alexandria
   module UI
@@ -171,14 +172,48 @@ module Alexandria
         @tid2 = nil
       end
 
+      def bg
+        @scanner_background
+      end
+
     end
 
+    require 'thread'
+    require 'monitor'
+
+    # assists in turning on progress bar when searching
+    # and turning it off when all search threads have completed...
+    class SearchThreadCounter < Monitor
+      
+      attr_reader :count
+
+      def initialize
+        @count = 0
+        super
+      end
+
+      def new_search
+        synchronize do
+          @count += 1
+        end
+      end
+
+      def end_search
+        synchronize do
+          @count -= 1 unless (@count == 0)
+        end
+      end
+
+    end
+
+
+
 
     class AcquireDialog < GladeBase
       include GetText
       include Logging
       extend GetText
-      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
       def initialize(parent, selected_library=nil, &block)
         super('acquire_dialog.glade')
@@ -196,17 +231,62 @@ module Alexandria
         setup_scanner_area
         init_treeview
         @book_results = Hash.new
+        
+        @search_thread_counter = SearchThreadCounter.new
+        @search_threads_running = @search_thread_counter.new_cond
+
+      end
+
+      def book_in_library(isbn10, library)
+        begin
+          isbn13 = Library.canonicalise_ean(isbn10)
+          # puts "new book #{isbn10} (or #{isbn13})"
+          match = library.find do |book|
+            # puts "testing #{book.isbn}"
+            (book.isbn == isbn10 || book.isbn == isbn13)
+            #puts "book #{book.isbn}"
+            #book == new_book 
+          end
+          # puts "book_in_library match #{match.inspect}"
+          (not match.nil?)
+        rescue Exception => ex
+          log.warn { "Failed to check for book #{isbn10} in library #{library}" }
+          true
+        end
       end
 
       def on_add
         model = @barcodes_treeview.model
+
+        libraries = Libraries.instance.all_libraries
+        library, is_new_library =
+          @combo_libraries.selection_from_libraries(libraries)
+
+        # NOTE at this stage, the ISBN is 10-digit...
+        #
+
         selection = @barcodes_treeview.selection
         isbns = []
+        isbn_duplicates = []
+
+        adding_a_selection = false
+
         if selection.count_selected_rows > 0
+
+          adding_a_selection = true
+
           model.freeze_notify do
             # capture isbns
             selection.selected_each do |model, path, iter|
-              isbns << iter[0]
+              isbn = iter[0]
+              if book_in_library(isbn, library)
+                isbn_duplicates << isbn
+              elsif isbns.include? isbn
+                log.info { "detected duplicate in scanned list #{isbn}" }
+                isbn_duplicates << isbn
+              else
+                isbns << isbn
+              end
             end
             # remove list items (complex, cf. tutorial...)
             # http://ruby-gnome2.sourceforge.jp/hiki.cgi?tut-treeview-model-remove
@@ -222,9 +302,28 @@ module Alexandria
             # try it this way... works because of persistent iters
             row_iters = []
             selection.selected_rows.each do |path|
-              row_iters << model.get_iter(path)
+              iter = model.get_iter(path)
+              isbn = iter[0]
+              if book_in_library(isbn, library)
+                log.debug { "#{isbn} is a duplicate" }
+              ##elsif isbns.include? isbn
+                # this won't work since multiple scans of the same
+                # book have the same isbn (so we can't refrain from removing
+                # one, we'd end up not removing any)
+                ## TODO add another column in the iter, like "isbn/01"
+                # that would allow this kind of behaviour...
+
+              elsif (not @book_results.has_key?(isbn)) #HAX
+                log.debug { "no book found for #{isbn}, not adding" }
+
+                # good enough for now
+              else
+                log.debug { "scheduling #{isbn} for removal from list" }
+                row_iters << iter
+              end
             end
             row_iters.each do |iter|
+              log.debug { "removing iter #{iter[0]}" }
               model.remove(iter)
             end
 
@@ -232,29 +331,75 @@ module Alexandria
         else
           model.freeze_notify do
             # capture isbns
+            row_iters = []
             model.each do |model, path, iter|
-              isbns << iter[0]
+              isbn = iter[0]
+              if (not @book_results.has_key?(isbn))
+                log.debug { "no book found for #{isbn}, not adding" }
+                adding_a_selection = true
+              elsif book_in_library(isbn, library)
+                log.info { "#{isbn} is a duplicate" }                
+                isbn_duplicates << isbn
+              elsif isbns.include? isbn
+                log.info { "detected duplicate in scanned list #{isbn}" }
+                isbn_duplicates << isbn
+              else
+                log.info { "scheduling #{isbn} for removal from list" }
+                isbns << isbn
+                row_iters << iter
+              end
             end
             # remove list items
-            model.clear
+            if (isbn_duplicates.empty? and (not adding_a_selection))
+              model.clear # TODO unless!!!
+              row_iters.clear
+            else
+              row_iters.each do |iter|
+                log.debug { "removing iter #{iter[0]}" }
+                model.remove(iter)
+              end
+            end
           end
         end
 
-        libraries = Libraries.instance.all_libraries
-        library, new_library =
-          @combo_libraries.selection_from_libraries(libraries)
+        books = []
+
         isbns.each do |isbn|
           log.debug { "Adding #{isbn}" }
           result = @book_results[isbn]
-          book = result[0]
-          cover_uri = result[1]
 
-          unless cover_uri.nil?
-            library.save_cover(book, cover_uri)
+          if result.nil?
+            # used to crash if book was not found in online lookup!
+            adding_a_selection = true
+            # ISBN not found, so keep it in the list
+            # TODO (for 0.6.5) should offer to add this book manually
+          else
+
+            book = result[0] 
+            cover_uri = result[1]
+            
+            unless cover_uri.nil?
+              library.save_cover(book, cover_uri)
+            end
+            books << book
+            library << book
+            library.save(book)
           end
-          library << book
-          library.save(book)
         end
+
+        if isbn_duplicates.empty?
+          @acquire_dialog.destroy unless adding_a_selection
+        else
+          message = n_("There was %d duplicate",
+                       "There were %d duplicates",
+                       isbn_duplicates.size) % isbn_duplicates.size
+          title = n_("Couldn't add this book",
+                     "Couldn't add these books",
+                     isbn_duplicates.size)
+          ErrorDialog.new(@parent, title, message)
+        end
+
+        @block.call(books, library, is_new_library)
       end
 
       def on_cancel
@@ -266,7 +411,7 @@ module Alexandria
 
       def read_barcode_scan
         @animator.start
-        log.debug { "reading CueCat data #{@scanner_buffer}" }
+        log.debug { "reading scanner data #{@scanner_buffer}" }
         barcode_text = nil
         isbn = nil
         begin
@@ -283,7 +428,7 @@ module Alexandria
         if isbn
           log.debug { "Got ISBN #{isbn}" }
           # TODO :: sound
-          play_sound("gnometris/turn")
+          play_sound("good_scan")
 
           @barcodes_treeview.model.freeze_notify do
             iter = @barcodes_treeview.model.append
@@ -295,20 +440,97 @@ module Alexandria
         else
           log.debug { "was not an ISBN barcode" }
           # TODO :: sound
-          play_sound("question")
+          play_sound("bad_scan")
+        end
+      end
+
+
+      # begin copy-n-paste from new_book_dialog
+
+      def notify_start_add_by_isbn
+        Gtk.idle_add do
+          main_progress_bar = MainApp.instance.appbar.children.first
+          main_progress_bar.visible = true
+          @progress_pulsing = Gtk.timeout_add(100) do
+            unless @destroyed
+              main_progress_bar.pulse
+              true
+            else
+              false
+            end
+          end
+          false
+        end
+      end
+
+      def notify_end_add_by_isbn
+        Gtk.idle_add do
+          MainApp.instance.appbar.children.first.visible = false
+          Gtk::timeout_remove(@progress_pulsing) if @progress_pulsing
+          false
+        end
+      end
+
+      def update(status, provider)
+        Gtk.idle_add do
+          messages = {
+            :searching => _("Searching Provider '%s'..."),
+            :error => _("Error while Searching Provider '%s'"),
+            :not_found => _("Not Found at Provider '%s'"),
+            :found => _("Found at Provider '%s'")
+          }
+          message = messages[status] % provider
+          log.debug { "update message : #{message}" }
+          # @parent.appbar.status = message
+          MainApp.instance.appbar.status = message # HACKish
+          false
+        end
+      end
+
+      # end copy-n-paste
+
+      private 
+      
+      def start_search         
+        @search_thread_counter.synchronize do 
+          if @search_thread_counter.count == 0
+            @search_thread_counter.new_search
+            @progress_bar_thread = Thread.new do
+              notify_start_add_by_isbn
+              Alexandria::BookProviders.instance.add_observer(self)
+              @search_thread_counter.synchronize do
+                @search_threads_running.wait_while do
+                  @search_thread_counter.count > 0
+                end
+              end
+              notify_end_add_by_isbn
+              Alexandria::BookProviders.instance.add_observer(self)
+            end
+          else
+            @search_thread_counter.new_search
+          end    
+        end
+      end
+
+
+      def stop_search
+        @search_thread_counter.synchronize do
+          @search_thread_counter.end_search
+          @search_threads_running.signal
         end
       end
 
-      private
 
       def lookup_book(isbn)
         lookup_thread = Thread.new(isbn) do |isbn|
           begin
+            start_search
             results = Alexandria::BookProviders.isbn_search(isbn)
             book = results[0]
             cover_uri = results[1]
             @book_results[isbn] = results
             set_cover_image_async(isbn, cover_uri)
+            # TODO add this as a block to Gtk.queue (needs new overall gui design)
             @barcodes_treeview.model.freeze_notify do
               iter = @barcodes_treeview.model.each do |model, path, iter|
                 if iter[0] == isbn
@@ -322,6 +544,8 @@ module Alexandria
           rescue StandardError => err
             log.error { "Book Search failed: #{err.message}"}
             log << err if log.error?
+          ensure
+            stop_search
           end
         end
       end
@@ -366,9 +590,29 @@ module Alexandria
         end
       end
 
+      def on_destroy
+        MainApp.instance.appbar.status = ""
+        notify_end_add_by_isbn
+        # TODO possibly make sure all threads have stopped running
+      end
+
       def setup_scanner_area
         @scanner_buffer = ""
-        @scanner = Alexandria::Scanners::CueCat.new # HACK :: use Registry
+        scanner_name = Alexandria::Preferences.instance.barcode_scanner
+        @scanner = Alexandria::Scanners::Registry.first # CueCat is default
+        Alexandria::Scanners::Registry.each do |scanner|
+          if scanner.name == scanner_name
+            @scanner = scanner
+          end
+        end
+
+        log.debug { "Using #{@scanner.name} scanner" }
+        message = _("Ready to use %s barcode scanner") % @scanner.name
+        MainApp.instance.appbar.status = message 
+
+        @prev_time = 0
+        @interval = 0
+
 
         @animator = BarcodeAnimation.new()
         @barcode_canvas.add(@animator)
@@ -402,6 +646,8 @@ module Alexandria
         @@debug_index = 0
         @scan_area.signal_connect("key-press-event") do |button, event|
           #log.debug { event.keyval }
+            # event.keyval == 65293 means Enter key
+          # HACK, this disallows numeric keypad entry of data...
           if event.keyval < 255
             if @scanner_buffer.empty?
               if event.keyval.chr == '`' # backtick key for devs
@@ -412,13 +658,53 @@ module Alexandria
                 log.debug { "Scanning! Received first character." }
               end
               # TODO :: sound
-              play_sound("iagno/flip-piece")
+              play_sound("scanning")
             end
             @scanner_buffer << event.keyval.chr
 
-            # or get event.keyval == 65293 meaning Enter key
+            # calculating average interval between input characters
+            if @prev_time.zero?
+              @prev_time = Time.now.to_f
+            else
+              now = Time.now.to_f
+              if @interval.zero?
+                @interval = now - @prev_time
+              else
+                new_interval = now - @prev_time
+                @interval = (@interval + new_interval) / 2.0
+              end
+              @prev_time = now
+            end
+
+            # if average interval is greater than around 45 milliseconds,
+            # then it's probably a human typing characters
+
             if @scanner.match? @scanner_buffer
-              read_barcode_scan
+              
+              Thread.new(@interval, @scanner_buffer) do |interval, buffer|
+                log.debug { "Waiting for more scanner input..." }
+                Gtk.idle_add do
+                  @animator.bg.fill_color_rgba = 0xFFF8C0FF
+                  false
+                end
+                time_to_wait = [3, interval*4].min
+                sleep(time_to_wait)
+                if buffer == @scanner_buffer
+                  log.debug { "Buffer unchanged; scanning complete" }
+                  Gtk.idle_add do
+                    @animator.bg.fill_color_rgba = 0xFFFFFFFF
+                    false
+                  end
+                  read_barcode_scan
+                  log.debug { "Avg interval between chars: #{@interval}" }
+                  @prev_time = 0
+                  @interval = 0
+
+                else
+                  log.debug { "Buffer has changed while waiting, reading more characters..." }
+                end
+              end
+
             end
           end
         end
@@ -495,8 +781,12 @@ module Alexandria
       end
 
       def play_sound(filename)
-        dir = "/usr/share/sounds"
-        Gnome::Sound.play("#{dir}/#{filename}.wav")
+        ## sound_file = "#{Config::SOUNDS_DIR}/#{filename}.ogg"
+        sound_file = "#{Config::SOUNDS_DIR}/#{filename}.wav"
+        log.info { "playing #{sound_file}" }
+        Gnome::Sound.play(sound_file)
+        # HACK, if your GNOME "system sounds" don't work, uncomment this...
+        ## `aplay #{sound_file}`
       end
 
     end
