@@ -276,25 +276,28 @@ module Alexandria
         @image_thread.kill if @image_thread
 
         notify_start_add_by_isbn
-        @find_thread = Thread.new do
-          log.info { "New @find_thread #{Thread.current}" }
-          begin
-            Alexandria::BookProviders.instance.add_observer(self)
-            @results = Alexandria::BookProviders.search(criterion, mode)
-            
-            log.info { "got #{@results.length} results" }
-          rescue => e
-            @find_error = e.message
-          ensure
-            Alexandria::BookProviders.instance.delete_observer(self)
-            #   notify_end_add_by_isbn
+        Gtk.idle_add do
+          @find_thread = Thread.new do
+            log.info { "New @find_thread #{Thread.current}" }
+            begin
+              Alexandria::BookProviders.instance.add_observer(self)
+              @results = Alexandria::BookProviders.search(criterion, mode)
+              
+              log.info { "got #{@results.length} results" }
+            rescue => e
+              @find_error = e.message
+            ensure
+              Alexandria::BookProviders.instance.delete_observer(self)
+              #notify_end_add_by_isbn
+            end
           end
+          false
         end
 
         Gtk.timeout_add(100) do
           # This block copies results into the tree view, or shows an
           # error if the search failed.
-
+          
           # Err... continue == false if @find_error
           continue = if @find_error
                        ErrorDialog.new(@parent,
@@ -384,38 +387,47 @@ module Alexandria
         assert_not_exist(library, @entry_isbn.text)        
         @button_add.sensitive = false  
         notify_start_add_by_isbn
-        @find_thread = Thread.new do
-          log.info { "New @find_thread #{Thread.current}" }
-          begin
-            # MAJOR HACK, add this again...
-            Alexandria::BookProviders.instance.add_observer(self)
-            book, cover_url = Alexandria::BookProviders.isbn_search(isbn)
-            # prov = FakeBookProviders.new()
-            # prov.add_observer(self)
-            # book, cover_url = prov.isbn_search(isbn)
+        Gtk.idle_add do
 
-            notify_end_add_by_isbn
+          @find_thread = Thread.new do
+            log.info { "New @find_thread #{Thread.current}" }
+            begin
+              # MAJOR HACK, add this again...
+              Alexandria::BookProviders.instance.add_observer(self)
+              book, cover_url = Alexandria::BookProviders.isbn_search(isbn)
+              # prov = FakeBookProviders.new()
+              # prov.add_observer(self)
+              # book, cover_url = prov.isbn_search(isbn)
 
-            if book
+              notify_end_add_by_isbn
 
-              puts "adding book #{book} to library"
-              add_book_to_library(library, book, cover_url)
-              @entry_isbn.text = ''
-              
-              post_addition([book], library, is_new)
-            else
-              post_addition([], library, is_new)
-            end
-          rescue => e
-            puts e.message
-            puts e.backtrace.join("\n> ")
-            @find_error = e.message
-            @button_add.sensitive = true
-            notify_end_add_by_isbn
-          ensure
-            Alexandria::BookProviders.instance.delete_observer(self)
-          end          
+              if book
+
+                puts "adding book #{book} to library"
+                add_book_to_library(library, book, cover_url)
+                @entry_isbn.text = ''
+                
+                post_addition([book], library, is_new)
+              else
+                post_addition([], library, is_new)
+              end
+            rescue => e
+              unless e.kind_of? Alexandria::BookProviders::NoResultsError
+                puts e.message
+                puts e.backtrace.join("\n> ")
+              end
+              @find_error = e.message
+              @button_add.sensitive = true
+              notify_end_add_by_isbn
+            ensure
+              Alexandria::BookProviders.instance.delete_observer(self)
+              notify_end_add_by_isbn
+            end          
+          end
+
+          false
         end
+
       end
 
       def add_selected_books(library, is_new)
@@ -538,7 +550,7 @@ module Alexandria
       end
 
       def update(status, provider)
-        Gtk.idle_add do
+        Gtk.queue do
           messages = {
             :searching => _("Searching Provider '%s'..."),
             :error => _("Error while Searching Provider '%s'"),
@@ -547,9 +559,10 @@ module Alexandria
           }
           message = messages[status] % provider
           log.debug { "update message : #{message}" }
+
           # @parent.appbar.status = message
           MainApp.instance.appbar.status = message # HACKish
-          false
+          #false
         end
       end
 
@@ -560,13 +573,18 @@ module Alexandria
             if Library.valid_isbn?(text) or Library.valid_ean?(text) or
                 Library.valid_upc?(text)
               Gtk.idle_add do
-                #puts "idly"
+
                 @entry_isbn.text = text
-                @button_add.grab_focus 
+                @entry_isbn.grab_focus
+                @entry_isbn.select_region(0, -1) # select all...
+                # @button_add.grab_focus 
                 false
               end
               log.debug { "Setting ISBN field to #{text}" }
               puts text # required, strangely, to prevent GUI strangeness
+              # above last checked with ruby-gnome2 0.17.1 2009-12-09
+              # if this puts is commented out, the cursor disappears
+              # from the @entry_isbn box... weird, ne? - CathalMagus
              end
           end
         end
